"""

PyLoadL: Python interface to IBM LoadLeveler

PyLoadL is a Pyrex wrapper that provides an interface 
to the IBM LoadLeveler batch scheduler API

"""

from distutils.core import setup
from distutils.extension import Extension
from distutils.command import clean
from distutils.sysconfig import get_python_lib
from Pyrex.Distutils import build_ext

import os, re
import sys, platform, popen2
from string import *
from stat import *

# Mininmum of Python 2.2.3 version required because that's what I've tested

if not hasattr(sys, 'version_info') or sys.version_info < (2,2,3,'final'):
   raise SystemExit, "Python 2.2.3 or later required to build Pyloadl."

# Get the OS type i.e Linux, AIX

OS = platform.system()
if OS == "AIX":

  print "Building for AIX"

  # Get the LoadLeveler version from AIX lslpp package utility

  pout,pin,perr= popen2.popen3('/usr/bin/lslpp -Ou -qlc LoadL.full',)
  pin.close()
  a = pout.readline()
  pout.close()
  b = perr.readline()
  perr.close()

  # Create a version string similar to the LoadLeveler PERL module alla Mike Hawkins
  # for creating LoadLeveler version dependent PyLoadL code

  LLVER = ''
  if ( not b ):
      c = a.split(':')
      ver_str = c[2].split('.')
      LLVER = LLVER + ver_str[0]
      for a in ver_str[1:]:
          if len(a) < 2:
              a = '0' + a
          LLVER = LLVER + a
  else:
      print "WARNING: lslpp error detected querying LoadLeveler installp package\n"
      print "%s" % b
      print "Exiting ...."
      sys.exit(1)

  # Standard LoadLeveler 3.x location under AIX

  include_dirs = ['/usr/lpp/LoadL/full/include','/usr/include']
  library_dirs = ['/usr/lpp/LoadL/full/lib','/usr/lib']
  libraries = ['llapi']

elif OS == "Linux":

  print "Building for linux"

  # if Linux then check for rpm

  pout,pin,perr= popen2.popen3('whereis -b rpm',)
  pin.close()
  a = pout.readline()
  pout.close()
  b = perr.readline()
  perr.close()

  RPM = ''
  if ( not b ):
     c = a.split(' ')
     for entry in c[1:]: 
        if entry[-1] == '\n': entry = entry[:-1]
        rpmfile = os.stat(entry)
        mode = rpmfile[ST_MODE]
        permissions = S_IMODE(mode)
	if S_ISREG(mode) and permissions & S_IEXEC:
          RPM = entry
  else:
     print "Error detected when searching for rpm binary"
     sys.exit(1)

  if not RPM:
     print "Error detected when searching for rpm binary"
     sys.exit(1)

  # Check if LoadLeveler is installed from the rpm database

  pout,pin,perr= popen2.popen3("%s -qa" % RPM,)
  pin.close()
  a = pout.readlines()
  pout.close()
  b = perr.readlines()
  perr.close()

  # Create a version string similar to the LoadLeveler PERL module alla Mike Hawkins
  # for creating LoadLeveler version dependent PyLoadL code

  if ( not b ):

    # If the LoadLeveler version is incorrectly detected set 
    # the appropiate version in LLVER i.e '03030101' = 3.3.1.1
    # by hand to force the correct version

    LLVER = ''
    if ( not LLVER ):
      for entry in a:
        if entry[-1] == '\n': entry = entry[:-1]

        package = re.compile(r'^LoadL-full-[^license]\w+\-\w+\-(?P<VERSION>\d+\.\d+\.\d+\.\d+)\-(?P<MIN>\d+)')
        line = package.match(entry)
        if line:

          if line.group("VERSION"):

            version = line.group("VERSION")
            ver_str = version.split('.')

            # RPM packaging format changed at 3.3.2.x
            # from 3.3.1.0-2 style to 3.3.2.2-0 style

            #print ver_str[0]+ver_str[1]+ver_str[2]
            if (ver_str[0]+ver_str[1]+ver_str[2]) < "332":
              #print "Old RPM package format"
              ver_str[3] = line.group("MIN")

            LLVER = LLVER + ver_str[0]
            for a in ver_str[1:]:
              if len(a) < 2:
                a = '0' + a
              LLVER = LLVER + a

  # Standard LoadLeveler 3.x location under Linux

  include_dirs = ['/opt/ibmll/LoadL/full/include', '/usr/include']
  library_dirs = ['/opt/ibmll/LoadL/full/lib','/usr/lib']
  libraries = ['llapi']
else:
  print "Sorry but %s is not a valid OS....exiting" % OS
  sys.exit(1)

if LLVER < '3010000':
    print "WARNING: This module only supports LoadLeveler 3.1 or higher\n"
    print "Exiting ...."
    sys.exit(1)
else:
    print "Detected LoadLeveler version %s" % LLVER

compiler_dir = os.path.join(get_python_lib(prefix=''), 'pyloadl/')

# Run the template files through the C preprocessor

if OS == "Linux":
  CPP = "cpp -D__linux__ -P"  # gcc not vacpp
  #CPP = "cc -D__linux__ -E"  # vacpp (IBM)
else:
  CPP = "cc -E"

os.system("%s -DLLVER=%s pyloadl/pyloadl.pyx.tmpl | grep -v '#line' | grep -v '# line' | grep -v '# 1' > pyloadl/pyloadl.pyx" % (CPP, LLVER))
os.system("%s -DLLVER=%s pyloadl/pyloadlvar.py.tmpl | grep -v '#line' | grep -v '# line' | grep -v '# 1' > pyloadl/pyloadlvar.py" % (CPP, LLVER))

# Trove classifiers

classifiers = """\
Development Status :: 4 - Beta
Intended Audience :: Developers
License :: OSI Approved :: GNU General Public License (GPL)
Natural Language :: English
Operating System :: POSIX :: AIX :: Linux
Programming Language :: Python
Topic :: Software Development :: Libraries :: Python Modules
"""

# Disutils fix for Python versions < 2.3 that didn't support classifiers
# as listed at http://www.python.org/~jeremy/weblog/030924.html

if sys.version_info < (2, 3):
    _setup = setup
    def setup(**kwargs):
       if kwargs.has_key("classifiers"):
          del kwargs["classifiers"]
       _setup(**kwargs)

doclines = __doc__.split("\n")

setup(
    name = "PyLoadL",
    version = "0.2.0",
    description = doclines[0],
    long_description = "\n".join(doclines[2:]),
    author = "Mark Roberts",
    author_email = "mark at gingergeeks co uk",
    url = "http://www.gingergeeks.co.uk/pyloadl/",
    classifiers = filter(None, classifiers.split("\n")),
    platforms = ["AIX","Linux"],
    keywords = ["IBM", "LoadLeveler"],
    packages = ["pyloadl"],
    ext_modules = [
        Extension( "pyloadl/pyloadl",["pyloadl/pyloadl.pyx"],
                   libraries = libraries,
                   library_dirs = library_dirs,
                   include_dirs = include_dirs )
    ],
    cmdclass = {"build_ext": build_ext}
)
